#include "linked_list.h"
#include <cassert>

//-----------------------------------------------------------------------------
sorted_wordlist::sorted_wordlist() : mBegin(nullptr), mEnd(nullptr)
{
}
//-----------------------------------------------------------------------------
sorted_wordlist::~sorted_wordlist()
{
   word_elem *we = mBegin;
   while (we)
   {
      we = mBegin->next();
      delete mBegin;
      mBegin = we;
   }
}
//-----------------------------------------------------------------------------
word &sorted_wordlist::add_elem(const word &elem)
{
   word_elem *we = mBegin;

   while (we)
   {
      if (!(we->get() < elem))
      {
         if (we->get() == elem)
         {
            we->get().inc_count();
         }
         else // we->get > elem
         {
            we = we->append(new word_elem(elem), false);
            if (!we->prev()) mBegin = we;
         }

         return we->get();
      }

      we = we->next();
   }

   if (mBegin)
   {
      we = mEnd = mEnd->append(new word_elem(elem));
   }
   else
   {
      we = mBegin = mEnd = new word_elem(elem);
   }

   return we->get();
}
//-----------------------------------------------------------------------------
sorted_wordlist::word_elem::word_elem(word *elem) :
   mElem(elem), mPrev(nullptr), mNext(nullptr)
{
}
//-----------------------------------------------------------------------------
sorted_wordlist::word_elem::word_elem(const word &elem) :
   mElem(new word(elem)), mPrev(nullptr), mNext(nullptr)
{

}
//-----------------------------------------------------------------------------
sorted_wordlist::word_elem::word_elem(word_elem *prev, word_elem *next, word *elem) :
   mElem(elem), mPrev(prev), mNext(next)
{
}
//-----------------------------------------------------------------------------
sorted_wordlist::word_elem::~word_elem()
{
   delete mElem;
   if (mPrev) mPrev->mNext = mNext;
   if (mNext) mNext->mPrev = mPrev;
}
//-----------------------------------------------------------------------------
sorted_wordlist::word_elem *sorted_wordlist::word_elem::next() const
{
   return mNext;
}
//-----------------------------------------------------------------------------
sorted_wordlist::word_elem *sorted_wordlist::word_elem::prev() const
{
   return mPrev;
}
//-----------------------------------------------------------------------------
word &sorted_wordlist::word_elem::get()
{
   return *mElem;
}
//-----------------------------------------------------------------------------
sorted_wordlist::word_elem *sorted_wordlist::word_elem::append(word_elem *elem,
                                                               bool post)
{
   assert(elem != nullptr);

   if (post)
   {
      if (mNext)
      {
         mNext->mPrev = elem;
         elem->mNext = mNext;
      }

      mNext       = elem;
      elem->mPrev = this;
   }
   else
   {
      if (mPrev)
      {
         mPrev->mNext = elem;
         elem->mPrev = mPrev;
      }
      elem->mNext = this;
      mPrev       = elem;
   }
   return elem;
}
//-----------------------------------------------------------------------------
